@@ -26,6 +26,8 @@ and comments related to this commit on GitHub.)
 """
 
 from .qpsolvers import (
+    Problem,
+    Solution,
     __version__,
     available_solvers,
     cvxopt_solve_qp,
@@ -48,6 +50,8 @@ from .qpsolvers import (
 )
 
 __all__ = [
+    "Problem",
+    "Solution",
     "__version__",
     "available_solvers",
     "cvxopt_solve_qp",
