@@ -20,6 +20,7 @@
 
 """Quadratic programming solvers in Python with a unified API"""
 
+from .exceptions import ProblemError
 from .problem import Problem
 from .solution import Solution
 from .solve_ls import solve_ls
@@ -43,10 +44,11 @@ from .solvers import (
 )
 from .utils import print_matrix_vector
 
-__version__ = "2.7.1"
+__version__ = "2.7.2"
 
 __all__ = [
     "Problem",
+    "ProblemError",
     "Solution",
     "__version__",
     "available_solvers",
