@@ -27,12 +27,16 @@ rooted in revisited primal-dual proximal algorithms. If you use ProxQP in some
 academic work, consider citing the corresponding paper [Bambade2022]_.
 """
 
+import warnings
 from typing import Optional, Union
 
 import numpy as np
 import scipy.sparse as spa
 from proxsuite import proxqp
 
+from ..problem import Problem
+from ..solution import Solution
+
 
 def __combine_inequalities(G, h, lb, ub, n: int, use_csc: bool):
     """
@@ -109,54 +113,21 @@ def __select_backend(backend: Optional[str], use_csc: bool):
     raise ValueError(f'Unknown ProxQP backend "{backend}')
 
 
-def proxqp_solve_qp(
-    P: Union[np.ndarray, spa.csc_matrix],
-    q: Union[np.ndarray, spa.csc_matrix],
-    G: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    h: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    A: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    b: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    lb: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    ub: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+def proxqp_solve_problem(
+    problem: Problem,
     initvals: Optional[np.ndarray] = None,
     verbose: bool = False,
     backend: Optional[str] = None,
     **kwargs,
-) -> Optional[np.ndarray]:
+) -> Solution:
     """
-    Solve a Quadratic Program defined as:
-
-    .. math::
-
-        \\begin{split}\\begin{array}{ll}
-        \\mbox{minimize} &
-            \\frac{1}{2} x^T P x + q^T x \\\\
-        \\mbox{subject to}
-            & G x \\leq h                \\\\
-            & A x = b                    \\\\
-            & lb \\leq x \\leq ub
-        \\end{array}\\end{split}
-
-    using `ProxQP <https://github.com/Simple-Robotics/proxsuite>`__.
+    Solve a Quadratic Program using `ProxQP
+    <https://github.com/Simple-Robotics/proxsuite>`__.
 
     Parameters
     ----------
-    P :
-        Positive semidefinite quadratic-cost matrix.
-    q :
-        Quadratic-cost vector.
-    G :
-        Linear inequality constraint matrix.
-    h :
-        Linear inequality constraint vector.
-    A :
-        Linear equality constraint matrix.
-    b :
-        Linear equality constraint vector.
-    lb :
-        Lower bound constraint vector.
-    ub :
-        Upper bound constraint vector.
+    problem :
+        Quadratic program to solve.
     initvals :
         Warm-start guess vector.
     backend :
@@ -168,7 +139,7 @@ def proxqp_solve_qp(
     Returns
     -------
     :
-        Solution to the QP, if found, otherwise ``None``.
+        Solution to the QP returned by the solver.
 
     Notes
     -----
@@ -222,6 +193,7 @@ def proxqp_solve_qp(
                 "Warm-start value specified in both `initvals` and `x` kwargs"
             )
         kwargs["x"] = initvals
+    P, q, G, h, A, b, lb, ub = problem.unpack()
     n: int = q.shape[0]
     use_csc: bool = (
         not isinstance(P, np.ndarray)
@@ -241,6 +213,89 @@ def proxqp_solve_qp(
         verbose=verbose,
         **kwargs,
     )
+    solution = Solution(problem)
+    solution.extras = {"info": result.info}
     if result.info.status != proxqp.QPSolverOutput.PROXQP_SOLVED:
-        return None
-    return result.x
+        return solution
+    solution.x = result.x
+    solution.y = result.y
+    if lb is not None or ub is not None:
+        solution.z = result.z[:-n]
+        solution.z_box = result.z[-n:]
+    else:  # lb is None and ub is None
+        solution.z = result.z
+    return solution
+
+
+def proxqp_solve_qp(
+    P: Union[np.ndarray, spa.csc_matrix],
+    q: Union[np.ndarray, spa.csc_matrix],
+    G: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    h: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    A: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    b: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    lb: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    ub: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    initvals: Optional[np.ndarray] = None,
+    verbose: bool = False,
+    backend: Optional[str] = None,
+    **kwargs,
+) -> Optional[np.ndarray]:
+    """
+    Solve a Quadratic Program defined as:
+
+    .. math::
+
+        \\begin{split}\\begin{array}{ll}
+        \\underset{\\mbox{minimize}}{x} &
+            \\frac{1}{2} x^T P x + q^T x \\\\
+        \\mbox{subject to}
+            & G x \\leq h                \\\\
+            & A x = b                    \\\\
+            & lb \\leq x \\leq ub
+        \\end{array}\\end{split}
+
+    using `ProxQP <https://github.com/Simple-Robotics/proxsuite>`__.
+
+    Parameters
+    ----------
+    P :
+        Positive semidefinite cost matrix.
+    q :
+        Cost vector.
+    G :
+        Linear inequality constraint matrix.
+    h :
+        Linear inequality constraint vector.
+    A :
+        Linear equality constraint matrix.
+    b :
+        Linear equality constraint vector.
+    lb :
+        Lower bound constraint vector.
+    ub :
+        Upper bound constraint vector.
+    initvals :
+        Warm-start guess vector.
+    backend :
+        ProxQP backend to use in ``[None, "dense", "sparse"]``. If ``None``
+        (default), the backend is selected based on the type of ``P``.
+    verbose :
+        Set to `True` to print out extra information.
+
+    Returns
+    -------
+    :
+        Primal solution to the QP, if found, otherwise ``None``.
+    """
+    warnings.warn(
+        "The return type of this function will change "
+        "to qpsolvers.Solution in qpsolvers v3.0",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    problem = Problem(P, q, G, h, A, b, lb, ub)
+    solution = proxqp_solve_problem(
+        problem, initvals, verbose, backend, **kwargs
+    )
+    return solution.x
