@@ -34,7 +34,7 @@ on installing this solver.
 """
 
 import warnings
-from typing import Any, List, Optional
+from typing import Any, List, Optional, Tuple
 
 import numpy as np
 from numpy import array, hstack, ones, vstack
@@ -44,6 +44,7 @@ from qpoases import PyQProblem as QProblem
 from qpoases import PyQProblemB as QProblemB
 from qpoases import PyReturnValue as ReturnValue
 
+from ..exceptions import ProblemError
 from ..problem import Problem
 from ..solution import Solution
 
@@ -107,6 +108,49 @@ def __prepare_options(
     return options
 
 
+def __convert_inequalities(
+    G: Optional[np.ndarray] = None,
+    h: Optional[np.ndarray] = None,
+    A: Optional[np.ndarray] = None,
+    b: Optional[np.ndarray] = None,
+) -> Tuple[np.ndarray, Optional[np.ndarray], np.ndarray]:
+    """
+    Convert linear constraints to qpOASES format.
+
+    Parameters
+    ----------
+    G :
+        Linear inequality constraint matrix.
+    h :
+        Linear inequality constraint vector.
+    A :
+        Linear equality constraint matrix.
+    b :
+        Linear equality constraint vector.
+
+    Returns
+    -------
+    :
+        Tuple ``(C, lb_C, ub_C)`` for qpOASES.
+    """
+    lb_C: Optional[np.ndarray] = None
+    C = np.array([])
+    if G is not None and h is not None:
+        if A is not None and b is not None:
+            C = vstack([G, A])
+            lb_C = hstack([-__infty__ * ones(h.shape[0]), b])
+            ub_C = hstack([h, b])
+        else:  # no equality constraint
+            C = G
+            ub_C = h
+    else:  # no inequality constraint
+        if A is not None and b is not None:
+            C = A
+            lb_C = b
+            ub_C = b
+    return C, lb_C, ub_C
+
+
 def qpoases_solve_problem(
     problem: Problem,
     initvals: Optional[np.ndarray] = None,
@@ -143,6 +187,9 @@ def qpoases_solve_problem(
 
     Raises
     ------
+    ProblemError :
+        If the problem is ill-formed in some way, for instance if some matrices
+        are not dense.
     ValueError :
         If ``predefined_options`` is not a valid choice.
 
@@ -185,21 +232,7 @@ def qpoases_solve_problem(
         print("qpOASES: note that warm-start values ignored by wrapper")
     P, q, G, h, A, b, lb, ub = problem.unpack()
     n = P.shape[0]
-    lb_C: Optional[np.ndarray] = None
-    C = np.array([])
-    if G is not None and h is not None:
-        if A is not None and b is not None:
-            C = vstack([G, A])
-            lb_C = hstack([-__infty__ * ones(h.shape[0]), b])
-            ub_C = hstack([h, b])
-        else:  # no equality constraint
-            C = G
-            ub_C = h
-    else:  # no inequality constraint
-        if A is not None and b is not None:
-            C = A
-            lb_C = b
-            ub_C = b
+    C, lb_C, ub_C = __convert_inequalities(G, h, A, b)
 
     args: List[Any] = []
     if C.shape[0] > 0:
@@ -213,7 +246,11 @@ def qpoases_solve_problem(
 
     options = __prepare_options(verbose, predefined_options, **kwargs)
     qp.setOptions(options)
-    return_value = qp.init(*args)
+
+    try:
+        return_value = qp.init(*args)
+    except TypeError as error:
+        raise ProblemError("problem has sparse matrices") from error
 
     solution = Solution(problem)
     if RET_INIT_FAILED <= return_value <= RET_INIT_FAILED_REGULARISATION:
@@ -306,6 +343,9 @@ def qpoases_solve_qp(
 
     Raises
     ------
+    ProblemError :
+        If the problem is ill-formed in some way, for instance if some matrices
+        are not dense.
     ValueError :
         If ``predefined_options`` is not a valid choice.
 
