@@ -37,6 +37,7 @@ import numpy as np
 import qpSWIFT
 
 from ..conversions import linear_from_box_inequalities, split_dual_linear_box
+from ..exceptions import ProblemError
 from ..problem import Problem
 from ..solution import Solution
 
@@ -69,6 +70,12 @@ def qpswift_solve_problem(
     :
         Solution returned by the solver.
 
+    Raises
+    ------
+    ProblemError :
+        If the problem is ill-formed in some way, for instance if some matrices
+        are not dense or the problem has no inequality constraint.
+
     Note
     ----
     **Rank assumptions:** qpSWIFT requires the QP matrices to satisfy the
@@ -127,7 +134,7 @@ def qpswift_solve_problem(
         print("qpSWIFT: warm-start values are ignored by wrapper")
     P, q, G, h, A, b, lb, ub = problem.unpack()
     if lb is not None or ub is not None:
-        G, h = linear_from_box_inequalities(G, h, lb, ub)
+        G, h = linear_from_box_inequalities(G, h, lb, ub, use_sparse=False)
     result: dict = {}
     kwargs.update(
         {
@@ -135,16 +142,19 @@ def qpswift_solve_problem(
             "VERBOSE": 1 if verbose else 0,
         }
     )
-    if G is not None and h is not None:
-        if A is not None and b is not None:
-            result = qpSWIFT.run(q, h, P, G, A, b, kwargs)
-        else:  # no equality constraint
-            result = qpSWIFT.run(q, h, P, G, opts=kwargs)
-    else:  # no inequality constraint
-        # See https://github.com/qpSWIFT/qpSWIFT/issues/2
-        raise NotImplementedError(
-            "QPs without inequality constraints are not handled by qpSWIFT"
-        )
+
+    try:
+        if G is not None and h is not None:
+            if A is not None and b is not None:
+                result = qpSWIFT.run(q, h, P, G, A, b, kwargs)
+            else:  # no equality constraint
+                result = qpSWIFT.run(q, h, P, G, opts=kwargs)
+        else:  # no inequality constraint
+            # See https://github.com/qpSWIFT/qpSWIFT/issues/2
+            raise ProblemError("problem has no inequality constraint")
+    except TypeError as error:
+        raise ProblemError("problem has sparse matrices") from error
+
     basic_info = result["basicInfo"]
     adv_info = result["advInfo"]
 
@@ -232,6 +242,12 @@ def qpswift_solve_qp(
     :
         Solution to the QP, if found, otherwise ``None``.
 
+    Raises
+    ------
+    ProblemError :
+        If the problem is ill-formed in some way, for instance if some matrices
+        are not dense or the problem has no inequality constraint.
+
     Note
     ----
     .. _qpSWIFT rank assumptions:
