@@ -33,6 +33,7 @@ from numpy import hstack, vstack
 from quadprog import solve_qp
 
 from ..conversions import linear_from_box_inequalities, split_dual_linear_box
+from ..exceptions import ProblemError
 from ..problem import Problem
 from ..solution import Solution
 
@@ -61,6 +62,12 @@ def quadprog_solve_problem(
     :
         Solution to the QP, if found, otherwise ``None``.
 
+    Raises
+    ------
+    ProblemError :
+        If the problem is ill-formed in some way, for instance if some matrices
+        are not dense.
+
     Note
     ----
     The quadprog solver only considers the lower entries of :math:`P`,
@@ -77,7 +84,7 @@ def quadprog_solve_problem(
     if initvals is not None and verbose:
         warnings.warn("warm-start values are ignored by quadprog")
     if lb is not None or ub is not None:
-        G, h = linear_from_box_inequalities(G, h, lb, ub)
+        G, h = linear_from_box_inequalities(G, h, lb, ub, use_sparse=False)
     qp_G = P
     qp_a = -q
     qp_C: Optional[np.ndarray] = None
@@ -95,27 +102,28 @@ def quadprog_solve_problem(
             qp_C = -G.T
             qp_b = -h
         meq = 0
+
     try:
         x, obj, xu, iterations, y, iact = solve_qp(
             qp_G, qp_a, qp_C, qp_b, meq, **kwargs
         )
-    except ValueError as e:
-        error = str(e)
-        if "matrix G is not positive definite" in error:
+    except TypeError as error:
+        raise ProblemError("problem has sparse matrices") from error
+    except ValueError as error:
+        error_message = str(error)
+        if "matrix G is not positive definite" in error_message:
             # quadprog writes G the cost matrix that we write P in this package
-            raise ValueError("matrix P is not positive definite") from e
-        if "no solution" in error:
+            raise ValueError("matrix P is not positive definite") from error
+        if "no solution" in error_message:
             return Solution(problem)
-        warnings.warn(f"quadprog raised a ValueError: {e}")
+        warnings.warn(f"quadprog raised a ValueError: {error_message}")
         return Solution(problem)
 
     solution = Solution(problem)
     solution.x = x
     solution.obj = obj
 
-    n = P.shape[0]
-    m = qp_C.shape[1] - meq if qp_C is not None else 0
-    z, ys, z_box = __convert_dual_multipliers(y, n, m, meq, lb, ub)
+    z, ys, z_box = __convert_dual_multipliers(y, meq, lb, ub)
     solution.y = ys
     solution.z = z
     solution.z_box = z_box
@@ -130,8 +138,6 @@ def quadprog_solve_problem(
 
 def __convert_dual_multipliers(
     y: np.ndarray,
-    n: int,
-    m: int,
     meq: int,
     lb: Optional[np.ndarray],
     ub: Optional[np.ndarray],
@@ -160,6 +166,12 @@ def __convert_dual_multipliers(
         Tuple of dual multipliers :code:`z, ys, z_box` corresponding
         respectively to linear inequalities, linear equalities, and box
         inequalities.
+
+    Raises
+    ------
+    ProblemError :
+        If the problem is ill-formed in some way, for instance if some matrices
+        are not dense.
     """
     z, ys, z_box = None, None, None
     if meq > 0:
