@@ -74,9 +74,9 @@ class TestSolveQP(unittest.TestCase):
         Returns
         -------
         P : numpy.ndarray
-            Symmetric quadratic-cost matrix .
+            Symmetric cost matrix .
         q : numpy.ndarray
-            Quadratic-cost vector.
+            Cost vector.
         G : numpy.ndarray
             Linear inequality matrix.
         h : numpy.ndarray
@@ -102,9 +102,9 @@ class TestSolveQP(unittest.TestCase):
         Returns
         -------
         P : scipy.sparse.csc_matrix
-            Symmetric quadratic-cost matrix .
+            Symmetric cost matrix.
         q : numpy.ndarray
-            Quadratic-cost vector.
+            Cost vector.
         G : scipy.sparse.csc_matrix
             Linear inequality matrix.
         h : numpy.ndarray
@@ -689,7 +689,7 @@ class TestSolveQP(unittest.TestCase):
                 if solver == "proxqp"
                 else 1e-8
             )
-            eq_tolerance = 5e-5 if solver == "osqp" else 1e-10
+            eq_tolerance = 1e-4 if solver == "osqp" else 1e-10
             ineq_tolerance = (
                 2e-4
                 if solver in ["osqp", "scs"]
